@@ -1,6 +1,6 @@
 use std::future::Future;
 
-use crate::{Endpoint, IntoResponse, Request};
+use crate::{Endpoint, IntoResponse, Request, Result};
 
 /// Endpoint for the [`after`](super::EndpointExt::after) method.
 pub struct After<E, F> {
@@ -16,16 +16,16 @@ impl<E, F> After<E, F> {
 }
 
 #[async_trait::async_trait]
-impl<E, F, Fut, R> Endpoint for After<E, F>
+impl<E, F, Fut, T> Endpoint for After<E, F>
 where
     E: Endpoint,
-    F: Fn(E::Output) -> Fut + Send + Sync,
-    Fut: Future<Output = R> + Send,
-    R: IntoResponse,
+    F: Fn(Result<E::Output>) -> Fut + Send + Sync,
+    Fut: Future<Output = Result<T>> + Send,
+    T: IntoResponse,
 {
-    type Output = R;
+    type Output = T;
 
-    async fn call(&self, req: Request) -> Self::Output {
+    async fn call(&self, req: Request) -> Result<Self::Output> {
         (self.f)(self.inner.call(req).await).await
     }
 }
